-- $Id: interface.vhd,v 1.4 2001/08/13 21:01:14 hwf1 Exp $
--
-- KCPSM Interface
-- This interface parses a UDP datagram. If it is a KCPSM instruction code,
-- stores it into the dual port program ram. If it is a KCPSM data, stores
-- it into the dual port data ram. After KCPSM has processed the datagram,
-- it echoes back it out.
-- 
-- Author: Henry Fu
-- (c) 2001 Washington University, Applied Research Lab
--
-- this is the actual Interface entity
--

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.std_logic_unsigned.all;

entity Interface is
  
  port (
    CLK            : in  std_logic;                       -- clock
    Reset_l        : in  std_logic;                       -- reset

    D_MOD_IN       : in  std_logic_vector (31 downto 0);  -- data
    DataEn_MOD_IN  : in  std_logic;                       -- data enable
    SOF_MOD_IN     : in  std_logic;                       -- start of frame
    EOF_MOD_IN     : in  std_logic;                       -- end of frame
    SOD_MOD_IN     : in  std_logic;                       -- start of datagram
    TCA_MOD_IN     : out std_logic;                       -- congestion control

    D_OUT_MOD      : out std_logic_vector (31 downto 0);  -- data
    DataEn_OUT_MOD : out std_logic;                       -- data enable
    SOF_OUT_MOD    : out std_logic;                       -- start of frame
    EOF_OUT_MOD    : out std_logic;                       -- end of frame
    SOD_OUT_MOD    : out std_logic;                       -- start of datagram
    TCA_OUT_MOD    : in  std_logic;                       -- congestion control

    KCPSM_INTERRUPT: out std_logic;                       -- KCPSM Interrupt
    KCPSM_READ_STROBE : in std_logic;
    KCPSM_WRITE_STROBE : in std_logic;
    KCPSM_PROG_BANK: out std_logic;
    KCPSM_DATA_BANK: out std_logic_vector (1 downto 0);
    KCPSM_PORT_ID  : in  std_logic_vector (7 downto 0); 
    KCPSM_OUTPUT   : in  std_logic_vector (7 downto 0);

    PROGRAM_ADDRB  : out std_logic_vector (7 downto 0);   -- pram address line
    PROGRAM_DIB    : out std_logic_vector (31 downto 0);  -- pram data in line
    PROGRAM_WEB    : out std_logic;                       -- pram write enable
    PROGRAM_ENB    : out std_logic;                       -- pram chip enable
    PROGRAM_DOB    : in  std_logic_vector (31 downto 0);  -- pram data out line
    DATARAM_ADDRB  : out std_logic_vector (7 downto 0);   -- dram address line
    DATARAM_DIB    : out std_logic_vector (31 downto 0);  -- dram data in line
    DATARAM_WEB    : out std_logic;                       -- dram write enable
    DATARAM_ENB    : out std_logic;                       -- dram chip enable
    DATARAM_DOB    : in  std_logic_Vector (31 downto 0)); -- dram data out line

end Interface;


architecture behavior of Interface is

  -----------------------------------------------------------------------------
  -- HEADER FIFO
  -----------------------------------------------------------------------------
  component fifo_63x36
    port (
      din: IN std_logic_VECTOR(35 downto 0);
      wr_en: IN std_logic;
      wr_clk: IN std_logic;
      rd_en: IN std_logic;
      rd_clk: IN std_logic;
      ainit: IN std_logic;
      dout: OUT std_logic_VECTOR(35 downto 0);
      full: OUT std_logic;
      empty: OUT std_logic);
  end component;

  -----------------------------------------------------------------------------
  -- UDP Echo
  -----------------------------------------------------------------------------
  component UDPEcho
    port (
      CLK            : in  std_logic;     -- clock
      Reset_l        : in  std_logic;     -- synchronous reset, active low
      D_MOD_IN       : in  std_logic_vector (31 downto 0);      -- cell data
      DataEn_MOD_IN  : in  std_logic;     -- data enable
      SOF_MOD_IN     : in  std_logic;     -- start of frame
      EOF_MOD_IN     : in  std_logic;     -- end of frame
      SOD_MOD_IN     : in  std_logic;     -- start of datagram
      TCA_MOD_IN     : out std_logic;     -- congestion control
      D_OUT_MOD      : out std_logic_vector (31 downto 0);      -- cell data
      DataEn_OUT_MOD : out std_logic;     -- data enable
      SOF_OUT_MOD    : out std_logic;     -- start of frame
      EOF_OUT_MOD    : out std_logic;     -- end of frame
      SOD_OUT_MOD    : out std_logic;     -- start of datagram
      TCA_OUT_MOD    : in  std_logic);    -- congestion control
  end component;

  -----------------------------------------------------------------------------
  -- input buffer
  -----------------------------------------------------------------------------
  signal data_in : std_logic_vector (31 downto 0);  -- data input
  signal dataen_in : std_logic;         -- data enable
  signal sof_in : std_logic;            -- start of frame
  signal eof_in : std_logic;            -- end of frame
  signal sod_in : std_logic;            -- start of datagram
  signal tca_out : std_logic;
  
  -----------------------------------------------------------------------------
  -- udpecho out buffer
  -----------------------------------------------------------------------------
  signal data_e : std_logic_vector (31 downto 0);  -- data input
  signal dataen_e : std_logic;         -- data enable
  signal sof_e : std_logic;            -- start of frame
  signal eof_e : std_logic;            -- end of frame
  signal sod_e : std_logic;            -- start of datagram
  signal tca_e : std_logic;

  -----------------------------------------------------------------------------
  -- fifo out buffer
  -----------------------------------------------------------------------------
  signal data_f : std_logic_vector (31 downto 0);
  signal dataen_f : std_logic;
  signal sof_f : std_logic;
  signal eof_f : std_logic;
  signal sod_f : std_logic;

  -----------------------------------------------------------------------------
  -- final buffer
  -----------------------------------------------------------------------------
  signal dataen_buf : std_logic;
  signal sof_buf : std_logic;
  signal eof_buf : std_logic;
  signal sod_buf : std_logic;

  -----------------------------------------------------------------------------
  -- intermediate buffer
  -----------------------------------------------------------------------------
  signal dataen_int : std_logic;
  signal sof_int : std_logic;
  signal eof_int : std_logic;
  signal sod_int : std_logic;

  -----------------------------------------------------------------------------
  -- output buffer
  -----------------------------------------------------------------------------
  signal data_out : std_logic_vector (31 downto 0);
  signal dataen_out : std_logic;
  signal sof_out : std_logic;
  signal eof_out : std_logic;
  signal sod_out : std_logic;

  -----------------------------------------------------------------------------
  -- fifo signal 
  -----------------------------------------------------------------------------
  signal fifo_din2 : std_logic_vector (35 downto 0);
  signal fifo_dout2 : std_logic_vector (35 downto 0);
  signal ainit : std_Logic;
  signal wr_en2 : std_logic;
  signal rd_en2 : std_Logic;
  signal full : std_logic;
  signal empty : std_logic;

  -----------------------------------------------------------------------------
  -- kcpsm signal 
  -----------------------------------------------------------------------------
  signal kcpsm_read_strobe_in : std_logic;
  signal kcpsm_write_strobe_in : std_logic;
  signal kcpsm_port_id_in : std_logic_vector (7 downto 0);
  signal kcpsm_output_in : std_logic_vector (7 downto 0);

  -----------------------------------------------------------------------------
  -- control signal
  -----------------------------------------------------------------------------
  signal addr_count : unsigned (6 downto 0);   -- pram address counter
  signal bank_count : unsigned (0 downto 0);   -- pram address bank counter
  signal addrd_count : unsigned (5 downto 0);  -- dram address counter (rd)
  signal bankd_count : unsigned (1 downto 0);  -- dram address bank counter (rd)
  signal prog_bank : unsigned (0 downto 0);    -- KCPSM pram address bank 
  signal data_bank : unsigned (1 downto 0);    -- KCPSM dram address bank
  signal addrd2_count : unsigned (5 downto 0); -- dram address counter (wr)
  signal bankd2_count : unsigned (1 downto 0); -- dram address bank counter (wr)
  signal payload_count : unsigned (5 downto 0);
  signal write_queue_count : unsigned (1 downto 0);
  signal program_queue_count : unsigned (0 downto 0);
  signal enb : std_logic;
  signal web : std_logic;
  signal k_int2 : std_logic;
  signal enbd : std_logic;
  signal enbd2 : std_logic;
  signal webd : std_logic;
  signal is_start : std_logic;
  signal is_data : std_logic;
  signal is_program : std_logic;
  signal is_nopayload : std_logic;
  signal wr_grant : std_logic;
  signal wr_done : std_logic;
  signal fifo2udp : std_logic;
  signal ram2udp : std_logic;
  signal is_loaded : std_logic;
  signal is_reload : std_logic;

  -----------------------------------------------------------------------------
  -- udp to fifo state machine
  -----------------------------------------------------------------------------
  type UDP2FIFO_StateType is (Idle, UDP2FIFO, Done, Done2); -- states
  signal udp2fifo_state    : UDP2FIFO_StateType;  -- current udp2dpram state
  signal nx_udp2fifo_state : UDP2FIFO_StateType;  -- next udp2dpram state

  -----------------------------------------------------------------------------
  -- check data / program state machine
  -----------------------------------------------------------------------------
  type CHK_D_I_StateType is (Idle, Req, Req2, CHK_D_I, Program, Data, 
                             Nopayload);  
  signal chk_d_i_state    : CHK_D_I_StateType;  -- current udp2dpram state
  signal nx_chk_d_i_state : CHK_D_I_StateType;  -- next udp2dpram state

  -----------------------------------------------------------------------------
  -- udp to dpram state machine
  -----------------------------------------------------------------------------
  type FIFO2RAM_StateType is (Idle, Start, 
                              fifo2program_req, fifo2program_udp, 
                              fifo2program_chk, fifo2program_copy, 
                              fifo2program_go, fifo2program_go2, 
                              fifo2dataram_copy, fifo2dataram_go, 
                              fifo2dataram_go2, fifopassthru,
                              fifopassthru_go, fifopassthru_go2);
  signal fifo2ram_state    : FIFO2RAM_StateType;  -- current udp2dpram state
  signal nx_fifo2ram_state : FIFO2RAM_StateType;  -- next udp2dpram state

  -----------------------------------------------------------------------------
  -- fifo to udp state machine
  -----------------------------------------------------------------------------
  type FIFO2UDP_StateType is (Idle, Start, fifo2udp_copy,
                              fifo2udp_go, fifo2udp_go2);
  signal fifo2udp_state    : FIFO2UDP_StateType;  -- current udp2dpram state2
  signal nx_fifo2udp_state : FIFO2UDP_StateType;  -- next udp2dpram state

  -----------------------------------------------------------------------------
  -- ram to udp state machine
  -----------------------------------------------------------------------------
  type RAM2UDP_StateType is (Idle, ram2udp_copy, ram2udp_copy2,
                             ram2udp_sof, ram2udp_ip_header,
                             ram2udp_ip_header2, ram2udp_ip_header3,
                             ram2udp_ip_header4, ram2udp_ip_header5,
                             ram2udp_udp_header, ram2udp_udp_header2,
                             ram2udp_payload, ram2udp_trail_copy,
                             ram2udp_trail_copy2, ram2udp_trailer,
                             ram2udp_trailer2, ram2udp_done);
  signal ram2udp_state    : RAM2UDP_StateType;  -- current udp2dpram state2
  signal nx_ram2udp_state : RAM2UDP_StateType;  -- next udp2dpram state
  signal pv_ram2udp_state : RAM2UDP_StateType;  -- previous udp2dpram state

  -----------------------------------------------------------------------------
  -- reset KCPSM state machine after data write 
  -----------------------------------------------------------------------------
  type RESET_KCPSM_StateType is (Idle, Check, K_Go1, K_Go2, NW_Check, NW_Go1,
                                 NW_Go2);
  signal reset_kcpsm_state    : RESET_KCPSM_StateType;  
  signal nx_reset_kcpsm_state : RESET_KCPSM_StateType; 

begin  -- behavior

  myfifo : fifo_63x36 port map (
    din         => fifo_din2,
    wr_en       => wr_en2,
    wr_clk      => clk,
    rd_en       => rd_en2,
    rd_clk      => clk,
    ainit       => ainit,
    dout        => fifo_dout2,
    full        => full,
    empty       => empty);

  myecho : udpecho port map (
    CLK            => clk, 
    Reset_l        => Reset_l, 
    D_MOD_IN       => D_MOD_IN, 
    DataEn_MOD_IN  => DataEn_MOD_IN, 
    SOF_MOD_IN     => SOF_MOD_IN, 
    EOF_MOD_IN     => EOF_MOD_IN, 
    SOD_MOD_IN     => SOD_MOD_IN, 
    TCA_MOD_IN     => TCA_MOD_IN, 
    D_OUT_MOD      => data_e, 
    DataEn_OUT_MOD => dataen_e,
    SOF_OUT_MOD    => sof_e, 
    EOF_OUT_MOD    => eof_e, 
    SOD_OUT_MOD    => sod_e,  
    TCA_OUT_MOD    => tca_e);

  -----------------------------------------------------------------------------
  -- purpose: copy data on clock edge
  -- type   : sequential
  -- inputs : CLK, Reset_l
  -- outputs: 
  -----------------------------------------------------------------------------
  clock_copy: process (CLK, Reset_l)
  begin  -- process clock_copy
    if CLK'event and CLK = '1' then  -- rising clock edge
      -- reset
      if Reset_l = '0' then
        sof_in  <= '0';
      else
        sof_in  <= sof_e;
      end if;

      -- input buffer stage
      data_in   <= data_e; 
      dataen_in <= dataen_e;
      eof_in    <= eof_e;
      sod_in    <= sod_e;
      tca_out   <= TCA_OUT_MOD; 

      -- fifo buffer stage 
      data_f   <= FIFO_DOUT2 (31 downto 0);
      dataen_f <= FIFO_DOUT2 (32);
      sof_f    <= FIFO_DOUT2 (33);
      eof_f    <= FIFO_DOUT2 (34);
      sod_f    <= FIFO_DOUT2 (35);      

      -- kcpsm buffer stage
      kcpsm_read_strobe_in <= KCPSM_READ_STROBE;
      kcpsm_write_strobe_in <= KCPSM_WRITE_STROBE;
      kcpsm_port_id_in <= KCPSM_PORT_ID;
      kcpsm_output_in <= KCPSM_OUTPUT;

      -- intermediate buffer stage
      dataen_int <= dataen_buf;
      sof_int <= sof_buf;
      eof_int <= eof_buf;
      sod_int <= sod_buf;

      -- final buffer stage
      data_out (31 downto 24) <= DATARAM_DOB (7 downto 0);
      data_out (23 downto 16) <= DATARAM_DOB (15 downto 8);
      data_out (15 downto 8) <= DATARAM_DOB (23 downto 16);
      data_out (7 downto 0) <= DATARAM_DOB (31 downto 24);
      dataen_out <= dataen_int;
      sof_out <= sof_int;
      eof_out <= eof_int;
      sod_out <= sod_int;
      
      -- state machines
      udp2fifo_state <= nx_udp2fifo_state;
      chk_d_i_state <= nx_chk_d_i_state;
      fifo2ram_state <= nx_fifo2ram_state;
      fifo2udp_state <= nx_fifo2udp_state;
      ram2udp_state <= nx_ram2udp_state;
      pv_ram2udp_state <= ram2udp_state;
      reset_kcpsm_state <= nx_reset_kcpsm_state;

    end if;
  end process clock_copy;

  -----------------------------------------------------------------------------
  -- purpose: statemachine to copy incoming data to fifo 
  -- type   : combinational
  -- inputs : udp2fifo_state, sof_in, eof_in, Resetl 
  -- outputs: nx_udp2fifo_state, wr_en2 
  -----------------------------------------------------------------------------
  udp2fifo_state_machine : process (Reset_l, udp2fifo_state, sof_in, eof_in,
                                    dataen_in, data_in)

  variable tmp_state : udp2fifo_statetype;
  variable tmp_wr_en2 : std_logic;

  begin -- process udp2fifo_state_machine

    -- default values
    tmp_state := udp2fifo_state;
    tmp_wr_en2 := '0';

    if Reset_l = '0' then
      tmp_state := Idle;
    else
      case udp2fifo_state is
        when Idle       => if sof_in = '1' then
                             tmp_state := udp2fifo;
                             tmp_wr_en2 := '1';
                           end if;
        when udp2fifo   => if eof_in = '1' then
                             tmp_state := Done;
                           end if;
                           tmp_wr_en2 := '1';
        when Done       => if dataen_in = '1' and data_in(15) = '1' then 
                             tmp_state := Done2;
                           end if;
                           tmp_wr_en2 := '1';
        when Done2      => if sof_in = '1' then
                             tmp_state := udp2fifo;
                           end if;
                           tmp_wr_en2 := '1';
      end case;
    end if;

    -- assign signals
    nx_udp2fifo_state <= tmp_state;
    wr_en2 <= tmp_wr_en2;

  end process udp2fifo_state_machine;

  -----------------------------------------------------------------------------
  -- purpose: statemachine to check if the incoming udp packet is data or inst 
  -- type   : combinational
  -- inputs : Reset_l, chk_d_i_state, sof_in, sod_in, data_in, eof_in
  -- outputs: nx_chk_d_i_state, is_data, is_program, is_nopaylaod
  -----------------------------------------------------------------------------
  chk_d_i_state_machine: process (Reset_l, chk_d_i_state, dataen_in,
                                  sof_in, sod_in, data_in, eof_in)

  variable tmp_state : CHK_D_I_StateType;
  variable tmp_is_start, tmp_is_data, tmp_is_program : std_logic;
  variable tmp_is_nopayload : std_logic;

  begin -- process chk_d_i_state_machine

    -- default values
    tmp_state := chk_d_i_state;
    tmp_is_data := is_data;
    tmp_is_program := is_program;
    tmp_is_nopayload := is_nopayload;
    tmp_is_start := '0';

    if Reset_l = '0' then
      tmp_state := Idle;
    else
      case chk_d_i_state is
        when Idle       => if sof_in = '1' then
                             tmp_state := Req;
                           end if;
        when Req        => if eof_in = '1' then
                             tmp_state := NoPayload;
                             tmp_is_start := '1';
                           elsif sod_in = '1' then
                             tmp_state := Req2;
                           end if;
        when Req2       => if eof_in = '1' then
                             tmp_state := NoPayload;
                           elsif dataen_in = '1' then 
                             tmp_state := CHK_D_I;
                           end if;
                           tmp_is_start := '1';
        when CHK_D_I    => if dataen_in = '1' and data_in = X"00000000" then
                             tmp_state := Program;
                           elsif dataen_in = '1' and data_in = X"00000001" then
                             tmp_state := Data; 
                           end if;
        when Program    => if eof_in = '1' then 
                             tmp_state := Idle;
                           end if;
                           tmp_is_program := '1';
                           tmp_is_data := '0';
                           tmp_is_nopayload := '0';
        when Data       => if eof_in = '1' then 
                             tmp_state := Idle;
                           end if;
                           tmp_is_program := '0';
                           tmp_is_data := '1';
                           tmp_is_nopayload := '0';
        when NoPayload  => tmp_state := Idle;
                           tmp_is_program := '0';
                           tmp_is_data := '0';
                           tmp_is_nopayload := '1';
      end case;
    end if;

    -- assign signals
    nx_chk_d_i_state <= tmp_state;
    is_start <= tmp_is_start;
    is_program <= tmp_is_program;
    is_data <= tmp_is_data;
    is_nopayload <= tmp_is_nopayload;

  end process chk_d_i_state_machine;   

  -----------------------------------------------------------------------------
  -- purpose: statemachine to copy incoming udp packet into ram 
  -- type   : combinational
  -- inputs : Reset_l, fifo2ram_state, is_start, is_program, is_data, 
  --          dataen_f, sof_f, sod_f, eof_f 
  -- outputs: nx_fifo2ram_state, rd_en2, enb, web, k_int, 
  --          enbd, webd, wr_grant
  -----------------------------------------------------------------------------
  fifo2ram_state_machine: process (Reset_l, fifo2ram_state, is_start, 
                                   is_program, is_data, dataen_f, 
                                   sof_f, sod_f, eof_f, data_f)

  variable tmp_state : FIFO2RAM_StateType;
  variable tmp_rd_en2, tmp_wr_grant : std_logic;
  variable tmp_enb, tmp_web, tmp_enbd, tmp_webd, tmp_is_loaded : std_logic;

  begin -- process fifo2ram_state_machine

    -- default values
    tmp_state := fifo2ram_state;
    tmp_rd_en2 := '0';
    tmp_enb := '0';
    tmp_web := '1';
    tmp_enbd := '0';
    tmp_webd := '1';
    tmp_wr_grant := '0';
    tmp_is_loaded := '0';

    if Reset_l = '0' then
      tmp_state := Idle;
    else
      case fifo2ram_state is
        when Idle               => if is_start = '1' then
                                     tmp_state := Start;
                                     tmp_rd_en2 := '1';
                                   end if;
        when Start              => if is_program = '1' and sof_f = '1' then
                                     tmp_state := fifo2program_req;
                                   elsif is_data = '1' and sof_f = '1' then
                                     tmp_state := fifo2dataram_copy; 
                                     tmp_enbd := '1';
                                     tmp_webd := '0';
                                   elsif is_nopayload = '1' and sof_f = '1' then
                                     tmp_state := fifopassthru;
                                   else
                                     tmp_wr_grant := '1';
                                   end if;
                                   tmp_rd_en2 := '1';
        when fifo2program_req   => if sod_f = '1' then
                                     tmp_state := fifo2program_udp;
                                   end if;
                                   tmp_rd_en2 := '1';
        when fifo2program_udp   => if dataen_f = '1' then 
                                     tmp_state := fifo2program_chk; 
                                   end if;
                                   tmp_rd_en2 := '1';
        when fifo2program_chk   => if dataen_f = '1' then 
                                     tmp_state := fifo2program_copy;
                                   end if;
                                   tmp_rd_en2 := '1';
        when fifo2program_copy  => if eof_f = '1' then
                                     tmp_state := fifo2program_go; 
                                   end if;
                                   if dataen_f = '1' then
                                     tmp_enb := '1';
                                     tmp_web := '0';
                                   end if;
                                   tmp_rd_en2 := '1';
        when fifo2program_go    => if dataen_f = '1' and data_f(15) = '1' then 
                                     tmp_state := fifo2program_go2;
                                   end if;
                                   tmp_rd_en2 := '1';
        when fifo2program_go2   => if dataen_f = '1' then 
                                     tmp_state := Start; 
                                     tmp_is_loaded := '1';
                                   end if;
                                   tmp_rd_en2 := '1';
        when fifo2dataram_copy  => if eof_f = '1' then
                                     tmp_state := fifo2dataram_go;
                                   end if;
                                   if dataen_f = '1' then
                                     tmp_enbd := '1';
                                     tmp_webd := '0';
                                   end if;
                                   tmp_rd_en2 := '1';
        when fifo2dataram_go    => if dataen_f = '1' and data_f(15) = '1' then 
                                     tmp_state := fifo2dataram_go2;
                                   end if;
                                   if dataen_f = '1' then
                                     tmp_enbd := '1';
                                     tmp_webd := '0';
                                   end if;
                                   tmp_rd_en2 := '1';
        when fifo2dataram_go2   => if dataen_f = '1' then 
                                     tmp_state := Start;
                                     tmp_enbd := '1';
                                     tmp_webd := '0';
                                   end if;
                                   tmp_rd_en2 := '1';
        when fifopassthru       => if eof_f = '1' then
                                     tmp_state := fifopassthru_go;
                                   end if;
                                   tmp_rd_en2 := '1';
        when fifopassthru_go    => if dataen_f = '1' and data_f(15) = '1' then
                                     tmp_state := fifopassthru_go2;
                                   end if;
                                   tmp_rd_en2 := '1';
        when fifopassthru_go2   => if dataen_f = '1' then
                                     tmp_state := Start;
                                   end if;
                                   tmp_rd_en2 := '1'; 
      end case;
    end if;

    -- assign signals
    nx_fifo2ram_state <= tmp_state;
    rd_en2 <= tmp_rd_en2;
    enb <= tmp_enb;
    web <= tmp_web;
    enbd <= tmp_enbd;
    webd <= tmp_webd;
    wr_grant <= tmp_wr_grant;
    is_loaded <= tmp_is_loaded;

  end process fifo2ram_state_machine; 

  -----------------------------------------------------------------------------
  -- purpose: control address counter 
  -- type   : sequential 
  -- inputs : Clk, Reset_l
  -- outputs: addr_count, bank_count, addrd_count, bankd_count, 
  -----------------------------------------------------------------------------
  addr_count_control : process (Clk, Reset_l) 
 
  begin -- process addr_count_control

    if CLK'event and CLK = '1' then
      if Reset_l = '0' then
        bank_count <= "1";      -- don't overwrite program initial content 
        bankd_count <= "00";    -- don't overwrite data initial content 
        addr_count <= "0000000";
        addrd_count <= "000000";
      else 
        case fifo2ram_state is
          when Start            => if is_data = '1' and sof_f = '1' then
                                     addrd_count <= addrd_count + 1;
                                   end if;
          when fifo2program_copy=> if dataen_f = '1' then 
                                     addr_count <= addr_count + 1;
                                   end if;
          when fifo2dataram_copy=> if dataen_f = '1' then 
                                     addrd_count <= addrd_count + 1; 
                                   end if;
          when fifo2program_go2 => addr_count <= "0000000";
                                   bank_count <= bank_count + 1;
          when fifo2dataram_go  => addrd_count <= addrd_count + 1;
          when fifo2dataram_go2 => addrd_count <= "000000";
                                   bankd_count <= bankd_count + 1;
          when others           => null;
        end case;
      end if;
    end if;

  end process addr_count_control;
 
  -----------------------------------------------------------------------------
  -- purpose: statemachine to reset KCPSM after a data write 
  -- type   : combinational
  -- inputs : Reset_l, reset_kcpsm_state, kcpsm_write_strobe_in, 
  --          kcpsm_write_strobe_in, bankd_count, data_bank,
  --          program_queue_count 
  -- outputs: nx_reset_kcpsm_state, k_int2, data_bank 
  -----------------------------------------------------------------------------
  reset_kcpsm_state_machine: process (Reset_l, reset_kcpsm_state, bankd_count,
                                      kcpsm_write_strobe_in, kcpsm_port_id_in,
                                      kcpsm_output_in,
                                      data_bank, program_queue_count)

  variable tmp_state : RESET_KCPSM_StateType;
  variable tmp_k_int2, tmp_is_reload : std_logic;

  begin -- process reset_kcpsm_state_machine

    -- default values
    tmp_state := reset_kcpsm_state;
    tmp_k_int2 := '0';
    tmp_is_reload := '0';

    if Reset_l = '0' then
      tmp_state := Idle;
    else
      case reset_kcpsm_state is
        when Idle       => if kcpsm_write_strobe_in = '1' and 
                              kcpsm_port_id_in = "11111111" then
                             if kcpsm_output_in = "11111111" then       
                                  tmp_state := Check;
                             elsif kcpsm_output_in = "00000000" then
                                  tmp_state := NW_Check;
                             end if;
                           end if;
        when Check      => if data_bank + 1 /= bankd_count then
                             tmp_state := K_Go1;
                           end if;
        when NW_Check   => if data_bank /= bankd_count then
                             tmp_state := NW_Go1;
                           end if;
        when K_Go1      => tmp_state := K_Go2;
                           tmp_k_int2 := '1';
                           tmp_is_reload := '1';
        when K_Go2      => tmp_state := Idle;
                           tmp_k_int2 := '1';
        when NW_Go1     => tmp_state := NW_Go2;
                           tmp_k_int2 := '1';
                           tmp_is_reload := '1';
        when NW_Go2     => tmp_state := Idle;
                           tmp_k_int2 := '1';
      end case;
    end if;

    -- assign signals
    nx_reset_kcpsm_state <= tmp_state;
    k_int2 <= tmp_k_int2;
    is_reload <= tmp_is_reload;

  end process reset_kcpsm_state_machine;

  -----------------------------------------------------------------------------
  -- purpose: logic to update the KCPSM pram dram address bank counter
  -- type   : combinational
  -- inputs : Reset_l, Clk 
  -- outputs: data_bank  
  -----------------------------------------------------------------------------
  kcpsm_addr_count_control: process (Clk, Reset_l)

  begin -- process write_queue_count_control

    if CLK'event and CLK = '1' then
      if Reset_l = '0' then
        prog_bank <= "0";
        data_bank <= "00";
      else
        case reset_kcpsm_state is
          when K_Go1    => data_bank <= data_bank + 1;
                           prog_bank <= prog_bank + program_queue_count;
          when NW_Go1   => prog_bank <= prog_bank + program_queue_count;
          when others   => null;
        end case;
      end if;
    end if;
  end process kcpsm_addr_count_control;

  -----------------------------------------------------------------------------
  -- purpose: logic to update the program queue count 
  -- type   : combinational
  -- inputs : Reset_l, Clk 
  -- outputs: program_queue_count  
  -----------------------------------------------------------------------------
  program_queue_count_control: process (Clk, Reset_l)

  begin -- process program_queue_count_control

    if CLK'event and CLK = '1' then
      if Reset_l = '0' then
        program_queue_count <= "0";
      elsif is_loaded = '1' then
        program_queue_count <= program_queue_count + 1; 
      elsif is_reload = '1' then
        program_queue_count <= "0"; 
      end if;
    end if;
  end process program_queue_count_control;

  -----------------------------------------------------------------------------
  -- purpose: logic to update the write queue count 
  -- type   : combinational
  -- inputs : Reset_l, Clk 
  -- outputs: write_queue_count  
  -----------------------------------------------------------------------------
  write_queue_count_control: process (Clk, Reset_l)

  begin -- process write_queue_count_control

    if CLK'event and CLK = '1' then
      if Reset_l = '0' then
        write_queue_count <= "00";
      elsif kcpsm_write_strobe_in = '1' and kcpsm_port_id_in = "11111111" then
        if kcpsm_output_in = "11111111" then
          write_queue_count <= write_queue_count + 1;
        end if;
      elsif wr_done = '1' then
        write_queue_count <= write_queue_count - 1;
      end if;
    end if;
  end process write_queue_count_control;

  -----------------------------------------------------------------------------
  -- purpose: statemachine to copy data from fifo to udp packet 
  -- type   : combinational
  -- inputs : Reset_l, fifo2udp_state, is_start, is_program, sof_f, eof_f 
  -- outputs: nx_fifo2udp_state
  -----------------------------------------------------------------------------
  fifo2udp_state_machine: process (Reset_l, fifo2udp_state, is_start, 
                                   is_program, sof_f, eof_f, dataen_f, data_f)

  variable tmp_state : FIFO2UDP_StateType;
  variable tmp_fifo2udp : std_logic;

  begin -- process fifo2udp_state_machine

    -- default values
    tmp_state := fifo2udp_state;
    tmp_fifo2udp := '0';

    if Reset_l = '0' then
      tmp_state := Idle;
    else
      case fifo2udp_state is
        when Idle               => if is_start = '1' then
                                     tmp_state := Start;
                                   end if;
        when Start              => if is_program = '1' and sof_f = '1' then
                                     tmp_state := fifo2udp_copy;
                                     tmp_fifo2udp := '1';
                                   elsif is_nopayload = '1' and sof_f = '1' then
                                     tmp_state := fifo2udp_copy;
                                     tmp_fifo2udp := '1';
                                   end if; 
        when fifo2udp_copy      => if eof_f = '1' then
                                     tmp_state := fifo2udp_go;
                                   end if;
                                   tmp_fifo2udp := '1';
        when fifo2udp_go        => if dataen_f = '1' and data_f(15) = '1' then 
                                     tmp_state := fifo2udp_go2;
                                   end if;
                                   tmp_fifo2udp := '1';
        when fifo2udp_go2       => tmp_state := Start;
                                   tmp_fifo2udp := '1';
      end case;
    end if;

  -- assign signals
  nx_fifo2udp_state <= tmp_state;
  fifo2udp <= tmp_fifo2udp;

  end process fifo2udp_state_machine;

  -----------------------------------------------------------------------------
  -- purpose: statemachine to copy data from ram to udp packet 
  -- type   : combinational
  -- inputs : Reset_l, ram2udp_state, is_start, KCPSM_WRITE_STROBE,
  --          KCPSM_PORT_ID, wr_grant, payload_count, data_out 
  -- outputs: nx_ram2udp_state, ram2udp
  -----------------------------------------------------------------------------
  ram2udp_state_machine: process (Reset_l, ram2udp_state, pv_ram2udp_state,
                                  write_queue_count,
                                  wr_grant, payload_count, data_out) 

  variable tmp_state : RAM2UDP_StateType;
  variable tmp_ram2udp, tmp_wr_done, tmp_enbd : std_logic;
  variable tmp_dataen_buf, tmp_sof_buf, tmp_eof_buf, tmp_sod_buf : std_logic;

  begin -- process ram2udp_state_machine

    -- default values
    tmp_state := ram2udp_state;
    tmp_ram2udp := '0';
    tmp_wr_done := '0';
    tmp_enbd := '0';
    tmp_dataen_buf := '0';
    tmp_sof_buf := '0';
    tmp_eof_buf := '0';
    tmp_sod_buf := '0';

    if Reset_l = '0' then
      tmp_state := Idle;
    elsif wr_grant = '1'then
      case ram2udp_state is
        when Idle               => if std_logic_vector(write_queue_count) /=
                                      "00" then
                                     tmp_state := ram2udp_sof;
                                   end if;
        when ram2udp_sof        => tmp_state := ram2udp_ip_header;
                                   tmp_enbd := '1';
                                   tmp_sof_buf := '1';
                                   tmp_ram2udp := '1';
        when ram2udp_ip_header  => tmp_state := ram2udp_ip_header2;
                                   tmp_enbd := '1';
                                   tmp_dataen_buf := '1';
                                   tmp_ram2udp := '1';
        when ram2udp_ip_header2 => tmp_state := ram2udp_ip_header3;
                                   tmp_enbd := '1';
                                   tmp_dataen_buf := '1';
                                   tmp_ram2udp := '1';
        when ram2udp_ip_header3 => tmp_state := ram2udp_ip_header4;
                                   tmp_enbd := '1';
                                   tmp_dataen_buf := '1';
                                   tmp_ram2udp := '1';
        when ram2udp_ip_header4 => tmp_state := ram2udp_ip_header5;
                                   tmp_enbd := '1';
                                   tmp_dataen_buf := '1';
                                   tmp_ram2udp := '1';
        when ram2udp_ip_header5 => tmp_state := ram2udp_udp_header;
                                   tmp_enbd := '1';
                                   tmp_dataen_buf := '1';
                                   tmp_ram2udp := '1';
        when ram2udp_udp_header => tmp_state := ram2udp_copy;
                                   tmp_enbd := '1';
                                   tmp_dataen_buf := '1';
                                   tmp_sod_buf := '1';
                                   tmp_ram2udp := '1';
        when ram2udp_copy       => tmp_state := ram2udp_copy2;
                                   tmp_enbd := '1';
                                   tmp_ram2udp := '1';
        when ram2udp_copy2      => if pv_ram2udp_state = ram2udp_copy then 
                                     tmp_state := ram2udp_udp_header2;
                                   else
                                     tmp_state := ram2udp_copy;
                                   end if;
                                   tmp_enbd := '1';
                                   tmp_ram2udp := '1'; 
        when ram2udp_udp_header2=> if pv_ram2udp_state = ram2udp_copy2 then 
                                     -- data line mapping
                                     -- check if packets is only 8 bytes long
                                     if data_out (31 downto 16) = X"0008" then 
                                       tmp_state := ram2udp_trail_copy;
                                     else
                                       tmp_state := ram2udp_payload;
                                     end if;
                                     tmp_dataen_buf := '1';
                                   else
                                     tmp_state := ram2udp_copy;
                                   end if;
                                   tmp_enbd := '1';
                                   tmp_ram2udp := '1';
        when ram2udp_payload    => if std_logic_vector (payload_count) = 
                                      "000011" then
                                     tmp_state := ram2udp_trail_copy;
                                     tmp_eof_buf := '1';
                                   end if;
                                   tmp_enbd := '1';
                                   tmp_dataen_buf := '1';
                                   tmp_ram2udp := '1';
        when ram2udp_trail_copy => tmp_state := ram2udp_trail_copy2;
                                   tmp_enbd := '1';
                                   tmp_ram2udp := '1';
        when ram2udp_trail_copy2=> if pv_ram2udp_state = ram2udp_trail_copy then
                                     tmp_state := ram2udp_trailer;
                                   else
                                     tmp_state := ram2udp_trail_copy;
                                   end if;
                                   tmp_enbd := '1';
                                   tmp_ram2udp := '1';
        when ram2udp_trailer    => if pv_ram2udp_state=ram2udp_trail_copy2 then
                                     if data_out (15) = '1' then
                                       tmp_state := ram2udp_trailer2; 
                                     else
                                       tmp_state := ram2udp_trail_copy;
                                     end if;
                                     tmp_dataen_buf := '1';
                                   else
                                     tmp_state := ram2udp_trail_copy;
                                   end if; 
                                   tmp_enbd := '1';
                                   tmp_ram2udp := '1';
        when ram2udp_trailer2   => tmp_state := ram2udp_done; 
                                   tmp_enbd := '1';
                                   tmp_dataen_buf := '1';
                                   tmp_ram2udp := '1';
        when ram2udp_done       => tmp_state := Idle;
                                   tmp_wr_done := '1';
      end case;
    end if;

  -- assign signals
  nx_ram2udp_state <= tmp_state;
  ram2udp <= tmp_ram2udp;
  wr_done <= tmp_wr_done;
  enbd2 <= tmp_enbd;
  dataen_buf <= tmp_dataen_buf;
  sof_buf <= tmp_sof_buf;
  eof_buf <= tmp_eof_buf;
  sod_buf <= tmp_sod_buf;

  end process ram2udp_state_machine;
 
  -----------------------------------------------------------------------------
  -- purpose: control write output address counter 
  -- type   : sequential 
  -- inputs : Clk, Reset_l
  -- outputs: addrd2_count, bankd2_count, 
  -----------------------------------------------------------------------------
  addr2_count_control : process (Clk, Reset_l) 
 
  begin -- process addr2_count_control

    if CLK'event and CLK = '1' then
      if Reset_l = '0' then
        bankd2_count <= "00";    
        addrd2_count <= "000000";
      elsif wr_grant = '1' then
        case ram2udp_state is
          when ram2udp_sof         => addrd2_count <= addrd2_count + 1;
          when ram2udp_ip_header   => addrd2_count <= addrd2_count + 1;
          when ram2udp_ip_header2  => addrd2_count <= addrd2_count + 1;
          when ram2udp_ip_header3  => addrd2_count <= addrd2_count + 1;
          when ram2udp_ip_header4  => addrd2_count <= addrd2_count + 1;
          when ram2udp_ip_header5  => addrd2_count <= addrd2_count + 1;
          when ram2udp_udp_header  => addrd2_count <= addrd2_count + 1;
          when ram2udp_udp_header2 => if pv_ram2udp_state = ram2udp_copy2 then
                                        addrd2_count <= addrd2_count + 1;
                                        payload_count <= 
                                        unsigned (data_out (23 downto 18));
                                      end if;
          when ram2udp_payload     => addrd2_count <= addrd2_count + 1;
                                      payload_count <= payload_count - 1;
          when ram2udp_trailer     => addrd2_count <= addrd2_count + 1;
          when ram2udp_trailer2    => addrd2_count <= addrd2_count + 1;
          when ram2udp_done        => addrd2_count <= "000000"; 
                                      bankd2_count <= bankd2_count + 1;
          when others              => null;
        end case;
      end if;
    end if;
  end process addr2_count_control;

  -----------------------------------------------------------------------------
  -- set address line
  -----------------------------------------------------------------------------
  PROGRAM_ADDRB (6 downto 0) <= std_logic_vector (addr_count);
  PROGRAM_ADDRB (7 downto 7) <= std_logic_vector (bank_count);
  DATARAM_ADDRB (5 downto 0) <= std_logic_vector (addrd_count) 
                                when ram2udp = '0' else
                                std_logic_vector (addrd2_count);
  DATARAM_ADDRB (7 downto 6) <= std_logic_vector (bankd_count)
                                when ram2udp = '0' else
                                std_logic_vector (bankd2_count);
  KCPSM_PROG_BANK <= std_logic (prog_bank(0));
  KCPSM_DATA_BANK <= std_logic_vector (data_bank); 

  -----------------------------------------------------------------------------
  -- set data line
  -----------------------------------------------------------------------------
  FIFO_DIN2 (31 downto 0) <= data_in;
  FIFO_DIN2 (32) <= dataen_in;
  FIFO_DIN2 (33) <= sof_in;
  FIFO_DIN2 (34) <= eof_in;
  FIFO_DIN2 (35) <= sod_in;
  PROGRAM_DIB <= data_f;
  DATARAM_DIB (31 downto 24) <= data_f (7 downto 0);
  DATARAM_DIB (23 downto 16) <= data_f (15 downto 8);
  DATARAM_DIB (15 downto 8) <= data_f (23 downto 16);
  DATARAM_DIB (7 downto 0) <= data_f (31 downto 24);

  -----------------------------------------------------------------------------
  -- set dpram control signal
  -----------------------------------------------------------------------------
  PROGRAM_ENB <= enb;
  PROGRAM_WEB <= web;
  DATARAM_ENB <= enbd when ram2udp = '0' else enbd2;
  DATARAM_WEB <= webd;

  KCPSM_INTERRUPT <= k_int2;  

  -----------------------------------------------------------------------------
  -- wire output
  -----------------------------------------------------------------------------
  D_OUT_MOD      <= data_f when fifo2udp = '1' else data_out;
  DataEn_OUT_MOD <= dataen_f when fifo2udp = '1' else dataen_out;
  SOF_OUT_MOD    <= sof_f when fifo2udp = '1' else sof_out;
  EOF_OUT_MOD    <= eof_f when fifo2udp = '1' else eof_out;
  SOD_OUT_MOD    <= sod_f when fifo2udp = '1' else sod_out;

  tca_e          <= tca_out;
  ainit          <= not(reset_l);

end behavior;


