-- $Id: module.vhd,v 1.5 2001/08/14 04:29:16 hwf1 Exp $
--
-- KCPSM Interface
-- This interface parses a UDP datagram. If it is a KCPSM instruction code,
-- stores it into the dual port program ram. If it is a KCPSM data, stores
-- it into the dual port data ram. After KCPSM has processed the datagram,
-- it echoes back it out.
-- 
-- Author: Henry Fu
-- (c) 2001 Washington University, Applied Research Lab
--
-- this file packages all necessary entities into an FPX module
--

library IEEE;
use IEEE.std_logic_1164.all;
library FPXLIB;
use FPXLIB.UDPProcessor.all;

ENTITY InterfaceModule IS
  PORT (
    -- Clock & Reset
    clk     : in STD_LOGIC;             -- 100MHz global clock
    reset_l : in STD_LOGIC;             -- Synchronous reset, asserted-low

    -- Enable & Ready
    -- Handshake for module reconfiguration.
    enable_l : in  STD_LOGIC;           -- Asserted low
    ready_l  : out STD_LOGIC;           -- Asserted low

    -- Cell Input Interface
    soc_mod_in   : in  STD_LOGIC;                      -- Start of cell
    d_mod_in     : in  STD_LOGIC_VECTOR(31 downto 0);  -- 32-bit data
    tca_mod_in   : out STD_LOGIC;                      -- Transmit cell available

    -- Cell Output Interface
    soc_out_mod   : out STD_LOGIC;                      -- Start of cell
    d_out_mod     : out STD_LOGIC_VECTOR(31 downto 0);  -- 32-bit data
    tca_out_mod   : in  STD_LOGIC;

    -- Test Data Output
    test_data     : out STD_LOGIC_VECTOR(31 downto 0));

end InterfaceModule;

architecture struc of InterfaceModule is

  component udpwrapper
    port (
      CLK             : in  std_logic;    -- clock
      Reset_l         : in  std_logic;    -- reset
      Enable_l        : in  std_logic;    -- enable
      Ready_l         : out std_logic;    -- ready
      SOC_MOD_IN      : in  std_logic;    -- start of cell
      D_MOD_IN        : in  std_logic_vector (31 downto 0);  -- data
      TCA_MOD_IN      : out std_logic;    -- transmit cell available
      D_OUT_APPL      : out std_logic_vector (31 downto 0);  -- data to appl
      DataEn_OUT_APPL : out std_logic;    -- data enable
      SOF_OUT_APPL    : out std_logic;    -- start of frame
      EOF_OUT_APPL    : out std_logic;    -- end of frame
      SOD_OUT_APPL    : out std_logic;    -- start of datagram
      TCA_OUT_APPL    : in  std_logic;    -- congestion control
      D_APPL_IN       : in  std_logic_vector (31 downto 0);  -- data from appl
      DataEn_APPL_IN  : in  std_logic;    -- data enable
      SOF_APPL_IN     : in  std_logic;    -- start of frame
      EOF_APPL_IN     : in  std_logic;    -- end of frame
      SOD_APPL_IN     : in  std_logic;    -- start of datagram
      TCA_APPL_IN     : out std_logic;    -- congestion control
      SOC_OUT_MOD     : out std_logic;    -- start of cell
      D_OUT_MOD       : out std_logic_vector (31 downto 0);  -- data
      TCA_OUT_MOD     : in  std_logic);   -- transmit cell available
  end component;

  -----------------------------------------------------------------------------
  -- Interface 
  -----------------------------------------------------------------------------
  component Interface
    port (
      CLK            : in  std_logic;                      -- clock
      Reset_l        : in  std_logic;                      -- reset

      D_MOD_IN       : in  std_logic_vector (31 downto 0); -- data
      DataEn_MOD_IN  : in  std_logic;                      -- data enable
      SOF_MOD_IN     : in  std_logic;                      -- start of frame
      EOF_MOD_IN     : in  std_logic;                      -- end of frame
      SOD_MOD_IN     : in  std_logic;                      -- start of datagram
      TCA_MOD_IN     : out std_logic;                      -- congestion control

      D_OUT_MOD      : out std_logic_vector (31 downto 0); -- data
      DataEn_OUT_MOD : out std_logic;                      -- data enable
      SOF_OUT_MOD    : out std_logic;                      -- start of frame
      EOF_OUT_MOD    : out std_logic;                      -- end of frame
      SOD_OUT_MOD    : out std_logic;                      -- start of datagram
      TCA_OUT_MOD    : in  std_logic;                      -- congestion control

      KCPSM_INTERRUPT: out std_logic;                      -- KCPSM Interrupt
      KCPSM_READ_STROBE : in std_logic;                    -- KCPSM read strobe
      KCPSM_WRITE_STROBE : in std_logic;
      KCPSM_PROG_BANK: out std_logic;                      -- KCPSM prog bank   
      KCPSM_DATA_BANK: out std_logic_vector (1 downto 0);  -- KCPSM data bank 
      KCPSM_PORT_ID  : in  std_logic_vector (7 downto 0);  -- KCPSM port id
      KCPSM_OUTPUT   : in  std_logic_vector (7 downto 0);

      PROGRAM_ADDRB  : out std_logic_vector (7 downto 0);  -- dpram address line
      PROGRAM_DIB    : out std_logic_vector (31 downto 0); -- dpram data in line
      PROGRAM_WEB    : out std_logic;                      -- dpram write enable
      PROGRAM_ENB    : out std_logic;                      -- dpram chip enable
      PROGRAM_DOB    : in  std_logic_vector (31 downto 0); -- dpram dataout line

      DATARAM_ADDRB  : out std_logic_vector (7 downto 0);
      DATARAM_DIB    : out std_logic_vector (31 downto 0);
      DATARAM_WEB    : out std_logic;
      DATARAM_ENB    : out std_logic;
      DATARAM_DOB    : in  std_logic_vector (31 downto 0));
  end component;

  -----------------------------------------------------------------------------
  -- KCPSM
  -----------------------------------------------------------------------------
  component kcpsm port
         (ADDR         : out std_logic_vector(7 downto 0);
          I            : in  std_logic_vector(15 downto 0);
          INPUT        : in  std_logic_vector(7 downto 0);
          OUTPUT       : out std_logic_vector(7 downto 0);
          PORT_ID      : out std_logic_vector(7 downto 0);
          READ_STROBE  : out std_logic;
          WRITE_STROBE : out std_logic;
          INTERRUPT    : in  std_logic;
          CLK          : in  std_logic);
  end component;

  -----------------------------------------------------------------------------
  -- DPRAM
  -----------------------------------------------------------------------------
  component program 
    port (
      addra: IN std_logic_VECTOR(8 downto 0);
      clka: IN std_logic;
      addrb: IN std_logic_VECTOR(7 downto 0);
      clkb: IN std_logic;
      dia: IN std_logic_VECTOR(15 downto 0);
      wea: IN std_logic;
      dib: IN std_logic_VECTOR(31 downto 0);
      web: IN std_logic;
      ena: IN std_logic;
      enb: IN std_logic;
      rsta: IN std_logic;
      rstb: IN std_logic;
      doa: OUT std_logic_VECTOR(15 downto 0);
      dob: OUT std_logic_VECTOR(31 downto 0));
  end component;

  -----------------------------------------------------------------------------
  -- DATARAM
  -----------------------------------------------------------------------------
  component dataram 
    port (
      addra: IN std_logic_VECTOR(9 downto 0);
      clka: IN std_logic;
      addrb: IN std_logic_VECTOR(7 downto 0);
      clkb: IN std_logic;
      dia: IN std_logic_VECTOR(7 downto 0);
      wea: IN std_logic;
      dib: IN std_logic_VECTOR(31 downto 0);
      web: IN std_logic;
      ena: IN std_logic;
      enb: IN std_logic;
      rsta: IN std_logic;
      rstb: IN std_logic;
      doa: OUT std_logic_VECTOR(7 downto 0);
      dob: OUT std_logic_VECTOR(31 downto 0));
  end component;

  -----------------------------------------------------------------------------
  -- signals kcpsm - dpram
  -----------------------------------------------------------------------------
  signal addra : std_logic_vector (8 downto 0);
  signal clka  : std_logic;
  signal dia   : std_logic_vector (15 downto 0);
  signal wea   : std_logic;
  signal ena   : std_logic;
  signal rsta  : std_logic;
  signal doa   : std_logic_vector (15 downto 0);


  -----------------------------------------------------------------------------
  -- signals interface - dpram
  -----------------------------------------------------------------------------
  signal addrb : std_logic_vector (7 downto 0);
  signal clkb  : std_logic;
  signal dib   : std_logic_vector (31 downto 0);
  signal web   : std_logic;
  signal enb   : std_logic;
  signal rstb  : std_logic;
  signal dob   : std_logic_vector (31 downto 0);

  -----------------------------------------------------------------------------
  -- signals kcpsm - dataram
  -----------------------------------------------------------------------------
  signal addrad : std_logic_vector (9 downto 0);
  signal clkad  : std_logic;
  signal diad   : std_logic_vector (7 downto 0);
  signal wead   : std_logic;
  signal enad   : std_logic;
  signal rstad  : std_logic;
  signal doad   : std_logic_vector (7 downto 0);

  -----------------------------------------------------------------------------
  -- signals interface - dataram
  -----------------------------------------------------------------------------
  signal addrbd : std_logic_vector (7 downto 0);
  signal clkbd  : std_logic;
  signal dibd   : std_logic_vector (31 downto 0);
  signal webd   : std_logic;
  signal enbd   : std_logic;
  signal rstbd  : std_logic;
  signal dobd   : std_logic_vector (31 downto 0);

  signal interrupt : std_logic;
  signal read_strobe : std_logic;
  signal nwead : std_logic;

  -----------------------------------------------------------------------------
  -- signals udpproc - udpecho
  -----------------------------------------------------------------------------
  signal data_u2h : std_logic_vector (31 downto 0);  -- data
  signal dataen_u2h : std_logic;                     -- data enable
  signal sof_u2h : std_logic;                        -- start of frame
  signal eof_u2h : std_logic;                        -- end of frame
  signal sod_u2h : std_logic;                        -- start of payload
  signal tca_u2h : std_logic;                        -- congestion control

  -----------------------------------------------------------------------------
  -- signals hellobob - udpproc
  -----------------------------------------------------------------------------
  signal data_h2u : std_logic_vector (31 downto 0);  -- data
  signal dataen_h2u : std_logic;                     -- data enable
  signal sof_h2u : std_logic;                        -- start of frame
  signal eof_h2u : std_logic;                        -- end of frame
  signal sod_h2u : std_logic;                        -- start of payload
  signal tca_h2u : std_logic;                        -- congestion control

begin  -- struc

  -----------------------------------------------------------------------------
  -- UDPWrapper 
  -----------------------------------------------------------------------------
  upw : UDPWrapper port map (
    CLK          => CLK,
    Reset_l      => reset_l,
    Enable_l     => enable_l,
    Ready_l      => ready_l,
    SOC_MOD_IN   => SOC_MOD_IN,
    D_MOD_IN     => D_MOD_IN,
    TCA_MOD_IN   => TCA_MOD_IN,
    SOC_OUT_MOD  => SOC_OUT_MOD,
    D_OUT_MOD    => D_OUT_MOD, -- open
    TCA_OUT_MOD  => TCA_OUT_MOD,
    D_OUT_APPL      => data_u2h,
    DataEn_OUT_APPL => dataen_u2h,
    SOF_OUT_APPL    => sof_u2h,
    EOF_OUT_APPL    => eof_u2h,
    SOD_OUT_APPL    => sod_u2h,
    TCA_OUT_APPL    => tca_u2h,
    D_APPL_IN       => data_h2u,
    DataEn_APPL_IN  => dataen_h2u,
    SOF_APPL_IN     => sof_h2u,
    EOF_APPL_IN     => eof_h2u,
    SOD_APPL_IN     => sod_h2u,
    TCA_APPL_IN     => tca_h2u);

  -----------------------------------------------------------------------------
  -- Interface
  -----------------------------------------------------------------------------
  inter : Interface port map (
    CLK            => CLK,
    Reset_l        => reset_l,
    D_MOD_IN       => data_u2h,
    DataEn_MOD_IN  => dataen_u2h,
    SOF_MOD_IN     => sof_u2h,
    EOF_MOD_IN     => eof_u2h,
    SOD_MOD_IN     => sod_u2h,
    TCA_MOD_IN     => tca_u2h,
    D_OUT_MOD      => data_h2u,
    DataEn_OUT_MOD => dataen_h2u,
    SOF_OUT_MOD    => sof_h2u,
    EOF_OUT_MOD    => eof_h2u,
    SOD_OUT_MOD    => sod_h2u,
    TCA_OUT_MOD    => tca_h2u,
    KCPSM_INTERRUPT=> interrupt,
    KCPSM_READ_STROBE => read_strobe,
    KCPSM_WRITE_STROBE => nwead,
    KCPSM_PROG_BANK=> addra(8),
    KCPSM_DATA_BANK=> addrad(9 downto 8),
    KCPSM_PORT_ID  => addrad(7 downto 0),
    KCPSM_OUTPUT   => diad,
    PROGRAM_ADDRB  => addrb,
    PROGRAM_DIB    => dib,
    PROGRAM_WEB    => web,
    PROGRAM_ENB    => enb,
    PROGRAM_DOB    => dob,
    DATARAM_ADDRB  => addrbd,
    DATARAM_DIB    => dibd,
    DATARAM_WEB    => webd,
    DATARAM_ENB    => enbd,
    DATARAM_DOB    => dobd);

  -----------------------------------------------------------------------------
  -- DPRAM
  -----------------------------------------------------------------------------
  myprogram : program port map (
    addra             => addra,
    clka              => clka,
    addrb             => addrb,
    clkb              => clkb,
    dia               => dia,
    wea               => wea,
    dib               => dib,
    web               => web,
    ena               => ena,
    enb               => enb,
    rsta              => rsta,
    rstb              => rstb,
    doa               => doa,
    dob               => dob);

  -----------------------------------------------------------------------------
  -- DATARAM
  -----------------------------------------------------------------------------
  mydataram : dataram port map (
    addra             => addrad, 
    clka              => clkad,
    addrb             => addrbd,
    clkb              => clkbd,
    dia               => diad,
    wea               => wead,
    dib               => dibd,
    web               => webd,
    ena               => enad,
    enb               => enbd,
    rsta              => rstad,
    rstb              => rstbd,
    doa               => doad,
    dob               => dobd);

  -----------------------------------------------------------------------------
  -- KCPSM
  -----------------------------------------------------------------------------
  cpu : kcpsm port map (
    addr         => addra(7 downto 0),  
    i            => doa,
    input        => doad,
    output       => diad,
    port_id      => addrad(7 downto 0),
    read_strobe  => read_strobe,
    write_strobe => nwead,
    interrupt    => interrupt,
    clk          => clk);

  -----------------------------------------------------------------------------
  -- dummy
  -----------------------------------------------------------------------------
  clka <= clk;
  dia <= (others => '0');
  wea <= '1';
  ena <= '1';
  rsta <= reset_l;

  clkb <= clk;
  rstb <= reset_l;  

  clkad <= clk;
  wead <= not(nwead);
  enad <= '1';
  rstad <= reset_l;

  clkbd <= clk;
  rstbd <= reset_l;

  test_data (31) <= sof_h2u;
  test_data (30) <= eof_h2u;
  test_data (29) <= sod_h2u;
  test_data (28) <= dataen_h2u;
  test_data (27 downto 16) <= data_h2u (11 downto 0);
  test_data (15) <= sof_u2h;
  test_data (14) <= eof_u2h;
  test_data (13) <= sod_u2h;
  test_data (12) <= dataen_u2h;
  test_data (11 downto 0) <= data_u2h (11 downto 0);

end struc;

-- synthesis translate_off
configuration interface_conf of InterfaceModule is
  for struc
    for all : UDPWrapper
      use configuration fpxlib.udpwrapper_conf;
    end for;
  end for;
end interface_conf;
-- synthesis translate_off
