-- $Id: udpecho.vhd,v 1.4 2001/08/07 21:45:14 hwf1 Exp $
--
-- KCPSM Interface
-- This interface parses a UDP datagram. If it is a KCPSM instruction code,
-- stores it into the dual port program ram. If it is a KCPSM data, stores
-- it into the dual port data ram. After KCPSM has processed the datagram,
-- it echoes back it out.
-- 
-- Author: Henry Fu 
-- (c) 2001 Washington University, Applied Research Lab
--
-- this module changes src/dest ip-address and port of a UDP datagram
--

library IEEE;
use IEEE.std_logic_1164.all;

entity UDPEcho is
  
  port (
    CLK            : in  std_logic;     -- clock
    Reset_l        : in  std_logic;     -- synchronous reset, active low
    D_MOD_IN       : in  std_logic_vector (31 downto 0);      -- cell data
    DataEn_MOD_IN  : in  std_logic;     -- data enable
    SOF_MOD_IN     : in  std_logic;     -- start of frame
    EOF_MOD_IN     : in  std_logic;     -- end of frame
    SOD_MOD_IN     : in  std_logic;     -- start of datagram
    TCA_MOD_IN     : out std_logic;     -- congestion control
    D_OUT_MOD      : out std_logic_vector (31 downto 0);      -- cell data
    DataEn_OUT_MOD : out std_logic;     -- data enable
    SOF_OUT_MOD    : out std_logic;     -- start of frame
    EOF_OUT_MOD    : out std_logic;     -- end of frame
    SOD_OUT_MOD    : out std_logic;     -- start of datagram
    TCA_OUT_MOD    : in  std_logic);    -- congestion control

end UDPEcho;


architecture behavior of UDPEcho is

  -----------------------------------------------------------------------------
  -- input signals
  -----------------------------------------------------------------------------
  signal data_in   : std_logic_vector (31 downto 0);            -- cell data
  signal dataen_in : std_logic;            -- data enable
  signal sof_in    : std_logic;            -- start of frame
  signal eof_in    : std_logic;            -- end of frame
  signal sod_in    : std_logic;            -- start of datagram

  -----------------------------------------------------------------------------
  -- previous data_in buffer 
  -----------------------------------------------------------------------------
  signal data_pv   : std_logic_vector (31 downto 0);            -- cell data
  signal dataen_pv : std_logic;            -- data enable
  signal sof_pv    : std_logic;            -- start of frame
  signal eof_pv    : std_logic;            -- end of frame
  signal sod_pv    : std_logic;            -- start of datagram

  -----------------------------------------------------------------------------
  -- next data_in buffer 
  -----------------------------------------------------------------------------
  signal data_nx   : std_logic_vector (31 downto 0);            -- cell data
  signal dataen_nx : std_logic;            -- data enable
  signal sof_nx    : std_logic;            -- start of frame
  signal eof_nx    : std_logic;            -- end of frame
  signal sod_nx    : std_logic;            -- start of datagram

  -----------------------------------------------------------------------------
  -- output signals
  -----------------------------------------------------------------------------
  signal data_out   : std_logic_vector (31 downto 0);           -- cell data
  signal dataen_out : std_logic;           -- data enable
  signal sof_out    : std_logic;           -- start of frame
  signal eof_out    : std_logic;           -- end of frame
  signal sod_out    : std_logic;           -- start of datagram

  -----------------------------------------------------------------------------
  -- state machine
  -----------------------------------------------------------------------------
  type StateType is (Idle, IPHdr1, IPHdr2, IPHdr3, SrcIPAddr, DestIPAddr, 
                     UDPPort); -- states
  signal state, nx_state : StateType;   -- current and new state

  
begin  -- behavior

  -----------------------------------------------------------------------------
  -- purpose: copy signals on clock edge
  -- type   : sequential
  -- inputs : CLK, Res,
  -- outputs: 
  -----------------------------------------------------------------------------
  clock_copy: process (CLK)
  begin  -- process clock_copy

    if CLK'event and CLK = '1' then     -- rising clock edge

      -- previous buffer
      data_pv <= D_MOD_IN;
      dataen_pv <= DataEn_MOD_IN;
      sof_pv <= SOF_MOD_IN;
      eof_pv <= EOF_MOD_IN;
      sod_pv <= SOD_MOD_IN;

      -- input buffer
      data_in <= data_pv;
      dataen_in <= dataen_pv;
      sof_in <= sof_pv;
      eof_in <= eof_pv;
      sod_in <= sod_pv;

      -- next buffer
      data_nx <= data_in;
      dataen_nx <= dataen_in;
      sof_nx <= sof_in;
      eof_nx <= eof_in;
      sod_nx <= sod_in;
      
      -- statemachine
      state <= nx_state;
      
    end if;

  end process clock_copy;

  -----------------------------------------------------------------------------
  -- purpose: state machine
  -- type   : combinational
  -- inputs : Reset_l, state, sof_in
  -- outputs: nx_state
  -----------------------------------------------------------------------------
  state_machine: process (Reset_l, state, sof_in, dataen_in, 
                          eof_in, sod_in, data_in)
    variable tmp_state : StateType;     -- new state
    variable tmp_data  : std_logic_vector (31 downto 0);
  begin  -- process state_machine

    -- default value
    tmp_state := state;
    tmp_data  := data_in;
    
    if Reset_l = '0' then
      tmp_state := Idle;
    else
      case state is
        when Idle       => if sof_in = '1' then
                             tmp_state := IPHdr1;
                           end if;
        when IPHdr1     => tmp_state := IPHdr2;
        when IPHdr2     => tmp_state := IPHdr3;
        when IPHdr3     => tmp_state := SrcIPAddr;
        when SrcIPAddr  => tmp_state := DestIPAddr;
                           tmp_data  := data_pv;
        when DestIPAddr => tmp_state := UDPPort;
                           tmp_data  := data_nx;
        when UDPPort    => if sod_in = '1' then 
                             tmp_data (31 downto 16) := data_in (15 downto 0);
                             tmp_data (15 downto 0)  := data_in (31 downto 16);
                           end if;
                           tmp_state := Idle;
      end case;
    end if;

    -- set state
    nx_state   <= tmp_state;
    data_out   <= tmp_data;
    dataen_out <= dataen_in;
    sof_out    <= sof_in;
    eof_out    <= eof_in;
    sod_out    <= sod_in;
    
  end process state_machine;

  -----------------------------------------------------------------------------
  -- wire output
  -----------------------------------------------------------------------------
  D_OUT_MOD <= data_out;
  DataEn_OUT_MOD <= dataen_out;
  SOF_OUT_MOD <= sof_out;
  EOF_OUT_MOD <= eof_out;
  SOD_OUT_MOD <= sod_out;
  TCA_MOD_IN  <= TCA_OUT_MOD;
  
end behavior;


