-- $Id: framewrapper.vhdl,v 1.2 2001/07/04 04:41:37 florian Exp $
--
-- FrameWrapper
-- The frame wrapper combines the cell processor and the frame processor for
-- convenience.
-- The frame processor abstracts from a cell based data stream. It handles AAL5
-- framing, AAL5-CRC and cell reassembly
-- 
-- Author: Florian Braun
-- (c) 2001 Washington University, Applied Research Lab
--

library IEEE;
use IEEE.std_logic_1164.all;
-- library fpxlib;

entity FrameWrapper is
  port (
    CLK             : in  std_logic;    -- clock
    Reset_l         : in  std_logic;    -- synchronous reset, active low

    Enable_l        : in  std_logic;    -- enable
    Ready_l         : out std_logic;    -- ready
    
    SOC_MOD_IN      : in  std_logic;    -- start of cell
    D_MOD_IN        : in  std_logic_vector (31 downto 0);  -- data
    TCA_MOD_IN      : out std_logic;    -- transmit cell available

    D_OUT_APPL      : out std_logic_vector (31 downto 0);  -- data
    DataEn_OUT_APPL : out std_logic;    -- data enable
    SOF_OUT_APPL    : out std_logic;    -- start of frame
    EOF_OUT_APPL    : out std_logic;    -- end of frame
    TCA_OUT_APPL    : in  std_logic;    -- transmit cell available

    D_APPL_IN       : in  std_logic_vector (31 downto 0);  -- data
    DataEn_APPL_IN  : in  std_logic;    -- data enable
    SOF_APPL_IN     : in  std_logic;    -- start of frame
    EOF_APPL_IN     : in  std_logic;    -- end of frame
    TCA_APPL_IN     : out std_logic;    -- transmit cell available

    SOC_OUT_MOD     : out std_logic;    -- start of cell
    D_OUT_MOD       : out std_logic_vector (31 downto 0);  -- data
    TCA_OUT_MOD     : in  std_logic);   -- transmit cell available
end FrameWrapper;



architecture struc of FrameWrapper is

  -----------------------------------------------------------------------------
  -- cell processor
  -----------------------------------------------------------------------------
  component CellProc
    port (
      CLK          : in  std_logic;
      Reset_l      : in  std_logic;
      Enable_l     : in  std_logic;
      Ready_l      : out std_logic;
      SOC_MOD_IN   : in  std_logic;
      D_MOD_IN     : in  std_logic_vector (31 downto 0);
      TCA_MOD_IN   : out std_logic;
      SOC_OUT_APPL : out std_logic;
      D_OUT_APPL   : out std_logic_vector (31 downto 0);
      TCA_OUT_APPL : in  std_logic;
      SOC_APPL_IN  : in  std_logic;
      D_APPL_IN    : in  std_logic_vector (31 downto 0);
      TCA_APPL_IN  : out std_logic;
      SOC_OUT_MOD  : out std_logic;
      D_OUT_MOD    : out std_logic_vector (31 downto 0);
      TCA_OUT_MOD  : in  std_logic);
  end component;

  -----------------------------------------------------------------------------
  -- frame processor
  -----------------------------------------------------------------------------
  component FrameProc
    port (
      CLK             : in  std_logic;
      Reset_l         : in  std_logic;
      Enable_l        : in  std_logic;
      Ready_l         : out std_logic;
      SOC_MOD_IN      : in  std_logic;
      D_MOD_IN        : in  std_logic_vector (31 downto 0);
      TCA_MOD_IN      : out std_logic;
      D_OUT_APPL      : out std_logic_vector (31 downto 0);
      DataEn_OUT_APPL : out std_logic;
      SOF_OUT_APPL    : out std_logic;
      EOF_OUT_APPL    : out std_logic;
      TCA_OUT_APPL    : in  std_logic;
      D_APPL_IN       : in  std_logic_vector (31 downto 0);
      DataEn_APPL_IN  : in  std_logic;
      SOF_APPL_IN     : in  std_logic;
      EOF_APPL_IN     : in  std_logic;
      TCA_APPL_IN     : out std_logic;
      SOC_OUT_MOD     : out std_logic;
      D_OUT_MOD       : out std_logic_vector (31 downto 0);
      TCA_OUT_MOD     : in  std_logic);
  end component;
  
  -----------------------------------------------------------------------------
  -- interconnection signals
  -----------------------------------------------------------------------------
  signal ready_cp : std_logic;          -- ready from cell processor
  signal ready_fp : std_logic;          -- ready from frame processor
  signal soc_c2f : std_logic;           -- soc from cp to fp
  signal data_c2f : std_logic_vector (31 downto 0);  -- data from cp to fp
  signal tca_c2f : std_logic;           -- tca from cp to fp
  signal soc_f2c : std_logic;           -- soc from fp to cp
  signal data_f2c : std_logic_vector (31 downto 0);  -- data from fp to cp
  signal tca_f2c : std_logic;           -- tca from fp to cp

begin  -- struc

  -----------------------------------------------------------------------------
  -- instantiate cell processor
  -----------------------------------------------------------------------------
  cp: CellProc
    port map (
      CLK          => CLK,
      Reset_l      => Reset_l,
      Enable_l     => Enable_l,
      Ready_l      => Ready_cp,
      SOC_MOD_IN   => SOC_MOD_IN,
      D_MOD_IN     => D_MOD_IN,
      TCA_MOD_IN   => TCA_MOD_IN,
      SOC_OUT_APPL => soc_c2f,
      D_OUT_APPL   => data_c2f,
      TCA_OUT_APPL => tca_c2f,
      SOC_APPL_IN  => soc_f2c,
      D_APPL_IN    => data_f2c,
      TCA_APPL_IN  => tca_f2c,
      SOC_OUT_MOD  => SOC_OUT_MOD,
      D_OUT_MOD    => D_OUT_MOD,
      TCA_OUT_MOD  => TCA_OUT_MOD);

  -----------------------------------------------------------------------------
  -- instantiate frame processor
  -----------------------------------------------------------------------------
  fp: FrameProc
    port map (
      CLK             => CLK,
      Reset_l         => Reset_l,
      Enable_l        => Enable_l,
      Ready_l         => Ready_fp,
      SOC_MOD_IN      => soc_c2f,
      D_MOD_IN        => data_c2f,
      TCA_MOD_IN      => tca_c2f,
      D_OUT_APPL      => D_OUT_APPL,
      DataEn_OUT_APPL => DataEn_OUT_APPL,
      SOF_OUT_APPL    => SOF_OUT_APPL,
      EOF_OUT_APPL    => EOF_OUT_APPL,
      TCA_OUT_APPL    => TCA_OUT_APPL,
      D_APPL_IN       => D_APPL_IN,
      DataEn_APPL_IN  => DataEn_APPL_IN,
      SOF_APPL_IN     => SOF_APPL_IN,
      EOF_APPL_IN     => EOF_APPL_IN,
      TCA_APPL_IN     => TCA_APPL_IN,
      SOC_OUT_MOD     => soc_f2c,
      D_OUT_MOD       => data_f2c,
      TCA_OUT_MOD     => tca_f2c);

  -----------------------------------------------------------------------------
  -- combine ready
  -----------------------------------------------------------------------------
  Ready_l <= not (ready_cp and ready_fp);
  
end struc;

-- synthesis translate_off
-- configuration framewrapper_conf of FrameWrapper is
--   for struc

--     for all : CellProc
--       use configuration fpxlib.cellproc_conf;
--     end for;
--     for all : FrameProc
--       use configuration fpxlib.frameproc_conf;
--     end for;
    
--   end for;
-- end framewrapper_conf;
-- synthesis translate_on
