-- $Id: ipwrapper.vhdl,v 1.1 2001/07/04 04:42:51 florian Exp $
--
-- IPWrapper
-- The IPWrapper combines the ip processor and the frame wrapper for
-- convenience.
-- The IP processor detects IP packets in the datastream, decrements the TTL
-- field and drops packets if the TTL field becomes zero. The user applications
-- gets the start of payload (SOP) signals when the actual payload begins.
-- 
-- Author: Florian Braun
-- (c) 2001 Washington University, Applied Research Lab
--

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
-- library fpxlib;

entity IPWrapper is
  
  port (
    CLK             : in  std_logic;    -- clock
    Reset_l         : in  std_logic;    -- reset

    Enable_l        : in  std_logic;    -- enable
    Ready_l         : out std_logic;    -- ready
    
    SOC_MOD_IN      : in  std_logic;    -- start of cell
    D_MOD_IN        : in  std_logic_vector (31 downto 0);  -- data
    TCA_MOD_IN      : out std_logic;    -- transmit cell available
    
    D_OUT_APPL      : out std_logic_vector (31 downto 0);  -- data to appl
    DataEn_OUT_APPL : out std_logic;    -- data enable
    SOF_OUT_APPL    : out std_logic;    -- start of frame
    EOF_OUT_APPL    : out std_logic;    -- end of frame
    SOP_OUT_APPL    : out std_logic;    -- start of payload
    TCA_OUT_APPL    : in  std_logic;    -- congestion control

    D_APPL_IN       : in  std_logic_vector (31 downto 0);  -- data from appl
    DataEn_APPL_IN  : in  std_logic;    -- data enable
    SOF_APPL_IN     : in  std_logic;    -- start of frame
    EOF_APPL_IN     : in  std_logic;    -- end of frame
    SOP_APPL_IN     : in  std_logic;    -- IP payload
    TCA_APPL_IN     : out std_logic;    -- congestion control
    
    SOC_OUT_MOD     : out std_logic;    -- start of cell
    D_OUT_MOD       : out std_logic_vector (31 downto 0);  -- data
    TCA_OUT_MOD     : in  std_logic);   -- transmit cell available
end IPWrapper;

architecture struc of IPWrapper is
  -----------------------------------------------------------------------------
  -- frame wrapper
  -----------------------------------------------------------------------------
  component FrameWrapper
    port (
      CLK             : in  std_logic;
      Reset_l         : in  std_logic;
      Enable_l        : in  std_logic;
      Ready_l         : out std_logic;
      SOC_MOD_IN      : in  std_logic;
      D_MOD_IN        : in  std_logic_vector (31 downto 0);
      TCA_MOD_IN      : out std_logic;
      D_OUT_APPL      : out std_logic_vector (31 downto 0);
      DataEn_OUT_APPL : out std_logic;
      SOF_OUT_APPL    : out std_logic;
      EOF_OUT_APPL    : out std_logic;
      TCA_OUT_APPL    : in  std_logic;
      D_APPL_IN       : in  std_logic_vector (31 downto 0);
      DataEn_APPL_IN  : in  std_logic;
      SOF_APPL_IN     : in  std_logic;
      EOF_APPL_IN     : in  std_logic;
      TCA_APPL_IN     : out std_logic;
      SOC_OUT_MOD     : out std_logic;
      D_OUT_MOD       : out std_logic_vector (31 downto 0);
      TCA_OUT_MOD     : in  std_logic);
  end component;
  
  -----------------------------------------------------------------------------
  -- ip processor
  -----------------------------------------------------------------------------
  component IPProc
    port (
      CLK             : in  std_logic;
      Reset_l         : in  std_logic;
      Enable_l        : in  std_logic;
      Ready_l         : out std_logic;
      D_MOD_IN        : in  std_logic_vector (31 downto 0);
      DataEn_MOD_IN   : in  std_logic;
      SOF_MOD_IN      : in  std_logic;
      EOF_MOD_IN      : in  std_logic;
      TCA_MOD_IN      : out std_logic;
      D_OUT_APPL      : out std_logic_vector (31 downto 0);
      DataEn_OUT_APPL : out std_logic;
      SOF_OUT_APPL    : out std_logic;
      EOF_OUT_APPL    : out std_logic;
      SOP_OUT_APPL    : out std_logic;
      TCA_OUT_APPL    : in  std_logic;
      D_APPL_IN       : in  std_logic_vector (31 downto 0);
      DataEn_APPL_IN  : in  std_logic;
      SOF_APPL_IN     : in  std_logic;
      EOF_APPL_IN     : in  std_logic;
      SOP_APPL_IN     : in  std_logic;
      TCA_APPL_IN     : out std_logic;
      D_OUT_MOD       : out std_logic_vector (31 downto 0);
      DataEn_OUT_MOD  : out std_logic;
      SOF_OUT_MOD     : out std_logic;
      EOF_OUT_MOD     : out std_logic;
      TCA_OUT_MOD     : in  std_logic);
  end component;

  -----------------------------------------------------------------------------
  -- interconnection signals
  -----------------------------------------------------------------------------
  signal ready_fw : std_logic;          -- ready_l from frame wrapper
  signal ready_ip : std_logic;          -- ready_l from ip processor
  -- frame wrapper to ip processor
  signal data_f2i          : std_logic_vector (31 downto 0);
  signal dataen_f2i        : std_logic;
  signal sof_f2i           : std_logic;
  signal eof_f2i           : std_logic;
  signal sop_f2i           : std_logic;
  signal tca_f2i           : std_logic;
  -- ip processor to frame wrapper
  signal data_i2f          : std_logic_vector (31 downto 0);
  signal dataen_i2f        : std_logic;
  signal sof_i2f           : std_logic;
  signal eof_i2f           : std_logic;
  signal sop_i2f           : std_logic;
  signal tca_i2f           : std_logic;

begin  -- struc

  -----------------------------------------------------------------------------
  -- instantiate framewrapper
  -----------------------------------------------------------------------------
  fw: FrameWrapper
    port map (
      CLK             => CLK,
      Reset_l         => Reset_l,
      Enable_l        => Enable_l,
      Ready_l         => Ready_fw,
      SOC_MOD_IN      => SOC_MOD_IN,
      D_MOD_IN        => D_MOD_IN,
      TCA_MOD_IN      => TCA_MOD_IN,
      D_OUT_APPL      => data_f2i,
      DataEn_OUT_APPL => dataen_f2i,
      SOF_OUT_APPL    => sof_f2i,
      EOF_OUT_APPL    => eof_f2i,
      TCA_OUT_APPL    => tca_f2i,
      D_APPL_IN       => data_i2f,
      DataEn_APPL_IN  => dataen_i2f,
      SOF_APPL_IN     => sof_i2f,
      EOF_APPL_IN     => eof_i2f,
      TCA_APPL_IN     => tca_i2f,
      SOC_OUT_MOD     => SOC_OUT_MOD,
      D_OUT_MOD       => D_OUT_MOD,
      TCA_OUT_MOD     => TCA_OUT_MOD);

  -----------------------------------------------------------------------------
  -- instantiate ip processor
  -----------------------------------------------------------------------------
  ip: IPProc
    port map (
      CLK             => CLK,
      Reset_l         => Reset_l,
      Enable_l        => Enable_l,
      Ready_l         => Ready_ip,

      D_MOD_IN        => data_f2i,
      DataEn_MOD_IN   => dataen_f2i,
      SOF_MOD_IN      => sof_f2i,
      EOF_MOD_IN      => eof_f2i,
      TCA_MOD_IN      => tca_f2i,

      D_OUT_APPL      => D_OUT_APPL,
      DataEn_OUT_APPL => DataEn_OUT_APPL,
      SOF_OUT_APPL    => SOF_OUT_APPL,
      EOF_OUT_APPL    => EOF_OUT_APPL,
      SOP_OUT_APPL    => SOP_OUT_APPL,
      TCA_OUT_APPL    => TCA_OUT_APPL,

      D_APPL_IN       => D_APPL_IN,
      DataEn_APPL_IN  => DataEn_APPL_IN,
      SOF_APPL_IN     => SOF_APPL_IN,
      EOF_APPL_IN     => EOF_APPL_IN,
      SOP_APPL_IN     => SOP_APPL_IN,
      TCA_APPL_IN     => TCA_APPL_IN,

      D_OUT_MOD       => data_i2f,
      DataEn_OUT_MOD  => dataen_i2f,
      SOF_OUT_MOD     => sof_i2f,
      EOF_OUT_MOD     => eof_i2f,
      TCA_OUT_MOD     => tca_i2f);

  -----------------------------------------------------------------------------
  -- combine ready
  -----------------------------------------------------------------------------
  Ready_l <= not (ready_fw and ready_ip);
  
end struc;

-- synthesis translate_off
-- configuration ipwrapper_conf of IPWrapper is
--   for struc

--     for all : FrameWrapper
--       use configuration fpxlib.framewrapper_conf;
--     end for;
--     for all : IPProc
--       use configuration fpxlib.ipproc_conf;
--     end for;
    
--   end for;
-- end ipwrapper_conf;
-- synthesis translate_on
