-- $Id: udpwrapper.vhdl,v 1.3 2001/07/17 22:45:36 florian Exp $
--
-- UDPWrapper
-- The UDP wrapper combines the IP wrapper and the UDP processor for
-- convenience.
-- The UDP processor checks IP packets for UDP packets and signals the start of
-- the datagram to the user application with the SOD signal.
-- 
-- Author: Florian Braun
-- (c) 2001 Washington University, Applied Research Lab
--

library IEEE;
use IEEE.std_logic_1164.all;
-- library fpxlib;

entity UDPWrapper is
  
  port (
    CLK             : in  std_logic;    -- clock
    Reset_l         : in  std_logic;    -- reset

    Enable_l        : in  std_logic;    -- enable
    Ready_l         : out std_logic;    -- ready
    
    SOC_MOD_IN      : in  std_logic;    -- start of cell
    D_MOD_IN        : in  std_logic_vector (31 downto 0);  -- data
    TCA_MOD_IN      : out std_logic;    -- transmit cell available

    D_OUT_APPL      : out std_logic_vector (31 downto 0);  -- data to appl
    DataEn_OUT_APPL : out std_logic;    -- data enable
    SOF_OUT_APPL    : out std_logic;    -- start of frame
    EOF_OUT_APPL    : out std_logic;    -- end of frame
    SOD_OUT_APPL    : out std_logic;    -- start of datagram
    TCA_OUT_APPL    : in  std_logic;    -- congestion control

    D_APPL_IN       : in  std_logic_vector (31 downto 0);  -- data from appl
    DataEn_APPL_IN  : in  std_logic;    -- data enable
    SOF_APPL_IN     : in  std_logic;    -- start of frame
    EOF_APPL_IN     : in  std_logic;    -- end of frame
    SOD_APPL_IN     : in  std_logic;    -- start of datagram
    TCA_APPL_IN     : out std_logic;    -- congestion control

    SOC_OUT_MOD     : out std_logic;    -- start of cell
    D_OUT_MOD       : out std_logic_vector (31 downto 0);  -- data
    TCA_OUT_MOD     : in  std_logic);   -- transmit cell available

end UDPWrapper;

architecture struc of UDPWrapper is

  -----------------------------------------------------------------------------
  -- ip wrapper
  -----------------------------------------------------------------------------
  component IPWrapper
    port (
      CLK             : in  std_logic;
      Reset_l         : in  std_logic;
      Enable_l        : in  std_logic;
      Ready_l         : out std_logic;
      SOC_MOD_IN      : in  std_logic;
      D_MOD_IN        : in  std_logic_vector (31 downto 0);
      TCA_MOD_IN      : out std_logic;
      D_OUT_APPL      : out std_logic_vector (31 downto 0);
      DataEn_OUT_APPL : out std_logic;
      SOF_OUT_APPL    : out std_logic;
      EOF_OUT_APPL    : out std_logic;
      SOP_OUT_APPL    : out std_logic;
      TCA_OUT_APPL    : in  std_logic;
      D_APPL_IN       : in  std_logic_vector (31 downto 0);
      DataEn_APPL_IN  : in  std_logic;
      SOF_APPL_IN     : in  std_logic;
      EOF_APPL_IN     : in  std_logic;
      SOP_APPL_IN     : in  std_logic;
      TCA_APPL_IN     : out std_logic;
      SOC_OUT_MOD     : out std_logic;
      D_OUT_MOD       : out std_logic_vector (31 downto 0);
      TCA_OUT_MOD     : in  std_logic);
  end component;

  -----------------------------------------------------------------------------
  -- udp processor
  -----------------------------------------------------------------------------
  component UDPProc
    port (
      CLK             : in  std_logic;
      Reset_l         : in  std_logic;
      Enable_l        : in  std_logic;
      Ready_l         : out std_logic;
      D_MOD_IN        : in  std_logic_vector (31 downto 0);
      DataEn_MOD_IN   : in  std_logic;
      SOF_MOD_IN      : in  std_logic;
      EOF_MOD_IN      : in  std_logic;
      SOP_MOD_IN      : in  std_logic;
      TCA_MOD_IN      : out std_logic;
      D_OUT_APPL      : out std_logic_vector (31 downto 0);
      DataEn_OUT_APPL : out std_logic;
      SOF_OUT_APPL    : out std_logic;
      EOF_OUT_APPL    : out std_logic;
      SOD_OUT_APPL    : out std_logic;
      TCA_OUT_APPL    : in  std_logic;
      D_APPL_IN       : in  std_logic_vector (31 downto 0);
      DataEn_APPL_IN  : in  std_logic;
      SOF_APPL_IN     : in  std_logic;
      EOF_APPL_IN     : in  std_logic;
      SOD_APPL_IN     : in  std_logic;
      TCA_APPL_IN     : out std_logic;
      D_OUT_MOD       : out std_logic_vector (31 downto 0);
      DataEn_OUT_MOD  : out std_logic;
      SOF_OUT_MOD     : out std_logic;
      EOF_OUT_MOD     : out std_logic;
      SOP_OUT_MOD     : out std_logic;
      TCA_OUT_MOD     : in  std_logic);
  end component;

  -----------------------------------------------------------------------------
  -- interconnecting signals
  -----------------------------------------------------------------------------
  signal ready_iw : std_logic;
  signal ready_up : std_logic;
  -- ip wrapper to udp processor
  signal data_i2u          : std_logic_vector (31 downto 0);
  signal dataen_i2u        : std_logic;
  signal sof_i2u           : std_logic;
  signal eof_i2u           : std_logic;
  signal sop_i2u           : std_logic;
  signal tca_i2u           : std_logic;
  -- udp processor to ip wrapper
  signal data_u2i          : std_logic_vector (31 downto 0);
  signal dataen_u2i        : std_logic;
  signal sof_u2i           : std_logic;
  signal eof_u2i           : std_logic;
  signal sop_u2i           : std_logic;
  signal tca_u2i           : std_logic;

begin  -- struc

  -----------------------------------------------------------------------------
  -- instantiate ip wrapper
  -----------------------------------------------------------------------------
  iw: IPWrapper
    port map (
      CLK             => CLK,
      Reset_l         => Reset_l,
      Enable_l        => Enable_l,
      Ready_l         => Ready_iw,
      
      SOC_MOD_IN      => SOC_MOD_IN,
      D_MOD_IN        => D_MOD_IN,
      TCA_MOD_IN      => TCA_MOD_IN,
      
      D_OUT_APPL      => data_i2u,
      DataEn_OUT_APPL => dataen_i2u,
      SOF_OUT_APPL    => sof_i2u,
      EOF_OUT_APPL    => eof_i2u,
      SOP_OUT_APPL    => sop_i2u,
      TCA_OUT_APPL    => tca_i2u,

      D_APPL_IN       => data_u2i,
      DataEn_APPL_IN  => dataen_u2i,
      SOF_APPL_IN     => sof_u2i,
      EOF_APPL_IN     => eof_u2i,
      SOP_APPL_IN     => sop_u2i,
      TCA_APPL_IN     => tca_u2i,
      
      SOC_OUT_MOD     => SOC_OUT_MOD,
      D_OUT_MOD       => D_OUT_MOD,
      TCA_OUT_MOD     => TCA_OUT_MOD);
  
  -----------------------------------------------------------------------------
  -- instantiate udp processor
  -----------------------------------------------------------------------------
  up : UDPProc
    port map (
      CLK             => CLK,
      Reset_l         => Reset_l,
      Enable_l        => Enable_l,
      Ready_l         => Ready_up,

      D_MOD_IN        => data_i2u,
      DataEn_MOD_IN   => dataen_i2u,
      SOF_MOD_IN      => sof_i2u,
      EOF_MOD_IN      => eof_i2u,
      SOP_MOD_IN      => sop_i2u,
      TCA_MOD_IN      => tca_i2u,

      D_OUT_APPL      => D_OUT_APPL,
      DataEn_OUT_APPL => DataEn_OUT_APPL,
      SOF_OUT_APPL    => SOF_OUT_APPL,
      EOF_OUT_APPL    => EOF_OUT_APPL,
      SOD_OUT_APPL    => SOD_OUT_APPL,
      TCA_OUT_APPL    => TCA_OUT_APPL,

      D_APPL_IN       => D_APPL_IN,
      DataEn_APPL_IN  => DataEn_APPL_IN,
      SOF_APPL_IN     => SOF_APPL_IN,
      EOF_APPL_IN     => EOF_APPL_IN,
      SOD_APPL_IN     => SOD_APPL_IN,
      TCA_APPL_IN     => TCA_APPL_IN,

      D_OUT_MOD       => data_u2i,
      DataEn_OUT_MOD  => dataen_u2i,
      SOF_OUT_MOD     => sof_u2i,
      EOF_OUT_MOD     => eof_u2i,
      SOP_OUT_MOD     => sop_u2i,
      TCA_OUT_MOD     => tca_u2i);

  -----------------------------------------------------------------------------
  -- combine ready_l
  -----------------------------------------------------------------------------
  Ready_l <= not (ready_iw and ready_up);
  
end struc;

-- synthesis translate_off
-- configuration udpwrapper_conf of UDPWrapper is
--   for struc

--     for all : IPWrapper
--       use configuration fpxlib.ipwrapper_conf;
--     end for;
--     for all : UDPProc
--       use configuration fpxlib.udpproc_conf;
--     end for;
    
--   end for;
-- end udpwrapper_conf;
-- synthesis translate_on
