/* A C program used to send and receive a UDP datagram */
/* There is an optional verbose mode [-v] */
/* By Henry Fu */
 
#include <stdio.h> 
#include <stdlib.h> 
#include <errno.h> 
#include <string.h> 
#include <netdb.h> 
#include <sys/types.h> 
#include <netinet/in.h> 
#include <sys/socket.h> 

#define MYPORT 64000   /* the source port */
#define TOPORT 7       /* the destination port */ 
#define MYSIZE 1536    /* the deault datagram size will be 1536 bytes */

/* Globals for getting command line arguments */
static char *options = "h:p:s:v";
static char *usage = "[-h destination host] [-p destination port] [-s source port] [-v]";
int vflg = 0, errflg = 0;
char *host;
int port = TOPORT;
int myport = MYPORT;
int bufsize = MYSIZE;
int size;
unsigned char *rbuf, *lbuf;
int i;
int x0, x1, x2, x3, y0, y1, y2, y3;

/* Required by getopt */
extern char *optarg;
extern int optind, opterr;

/* Function to get command line arguments */
void parseargs(int argc, char *argv[])
{
    int c;
    int ok = 1;

    while ((c=getopt(argc,argv,options)) != -1)
        switch (c)
        {
        case 'h':
          host = optarg;
          break;
        case 'p':
          port = atoi(optarg);
          break;
        case 's':
          myport = atoi(optarg);
          break;
        case 'v':
          vflg++;
          break;
        default:
          ok = 0;
        }
   
    if (!ok || optind < argc) {
      fprintf (stderr, "usage : %s %s\n", argv[0], usage);
      exit(0);
    }
}

/* Function to read input */
int parseinput()
{
    int length = 0;

    fputs ("Please enter the string you want to send: \n", stdout);
    fgets (&lbuf[4], bufsize, stdin);

    /* prepend KCPSM data packet header to begin of payload */
    lbuf[0] = (char) 0;
    lbuf[1] = (char) 0;
    lbuf[2] = (char) 0;
    lbuf[3] = (char) 1; 

    /* update payload size */ 
    length = strlen(&lbuf[4])+4; 

    /* replace the line feed with a null character */
    lbuf [length-1] = (char) 0;

    /* append "zeroes" to end of payload until the size is a multiple of 4 */
    while (length % 4 != 0) {
      lbuf[length] = (char) 0;
      length++;
    }

    return length;
}

int main(int argc, char *argv[])
{
    int sockfd;  
    struct hostent *he;
    struct sockaddr_in my_addr;
    struct sockaddr_in their_addr; /* connector's address information */

    lbuf = calloc(bufsize, sizeof(char));
    rbuf = calloc(bufsize, sizeof(char));
    host = calloc(80, sizeof(char));

    parseargs(argc, argv);
    size = parseinput();

    if (vflg) fprintf (stdout, "UDPStr is getting host name ...\n");

    if ((he=gethostbyname(host)) == NULL) {  /* get the host info */
        perror("UDPStr failed at gethostbyname");
        exit(1);
    }

    if (vflg) fprintf (stdout, "UDPStr is allocating socket ...\n");

    if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
        perror("UDPStr failed at socket");
        exit(1);
    }

    my_addr.sin_family = AF_INET;
    my_addr.sin_port = htons(myport);
    my_addr.sin_addr.s_addr = INADDR_ANY;
    bzero(&(my_addr.sin_zero), 8);

    if (vflg) fprintf (stdout, "UDPStr is binding ...\n");

    if (bind(sockfd, (struct sockaddr *)&my_addr, sizeof(struct sockaddr)) 
                                                                  == -1) {
        perror("UDPStr failed at bind");
        exit(1);
    }

    their_addr.sin_family = AF_INET;      /* host byte order */
    their_addr.sin_port = htons(port);    /* short, network byte order */
    their_addr.sin_addr = *((struct in_addr *)he->h_addr);
    bzero(&(their_addr.sin_zero), 8);     /* zero the rest of the struct */

    if (vflg) fprintf (stdout, "UDPStr is sending %d bytes ...\n", size);

    if (sendto(sockfd, lbuf, size, 0, 
               (const struct sockaddr *) &their_addr, sizeof(their_addr)) < 0) {
        perror("UDPStr failed at sendto");
        exit(1);
    }

    fprintf (stdout, "Sent %d bytes: \n", size);
    fprintf (stdout, "%s\n",&lbuf[4]);

    if ((size = read(sockfd, rbuf, bufsize)) < 0) {
        perror("UDPStr failed at read");
        exit(1);
    }

    if (vflg) fprintf (stdout, "UDPStr is receiving %d bytes ...\n", size);

    fprintf (stdout, "Received %d bytes: \n", size);
    fprintf (stdout, "%s\n",&rbuf[4]);

    close(sockfd);

    return 0;
}
